/**
 *  Copyright (C) 2010-2022 The Catrobat Team
 *  (http://developer.catrobat.org/credits)
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as
 *  published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 *
 *  An additional term exception under section 7 of the GNU Affero
 *  General Public License, version 3, is available at
 *  (http://developer.catrobat.org/license_additional_term)
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU Affero General Public License for more details.
 *
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see http://www.gnu.org/licenses/.
 */

#import "BaseLoginViewController.h"
#import "Pocket_Code-Swift.h"

@interface BaseLoginViewController ()
@property (nonatomic,assign) BOOL isMovedUp;
@end

@implementation BaseLoginViewController

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];

    self.isMovedUp = NO;
    self.edgesForExtendedLayout = UIRectEdgeNone;
    // Do any additional setup after loading the view.
    UITapGestureRecognizer *recognizer = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(dismissKeyboard)];
    [self.view addGestureRecognizer:recognizer];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#define kOFFSET_FOR_KEYBOARD 100.0

-(void)keyboardWillShow:(NSNotification*)notification {

    NSValue *keyboardValue = (notification.userInfo[UIKeyboardFrameEndUserInfoKey]);
    CGRect keyboardRect = keyboardValue.CGRectValue;
    
    UIScrollView* myScrollView;
    for (UIView *i in self.view.subviews){
        if([i isKindOfClass:[UIScrollView class]]){
            myScrollView = (UIScrollView *)i;
        }
    }
    
    myScrollView.scrollEnabled = true;
    UIEdgeInsets insets = UIEdgeInsetsMake(0.0, 0.0, keyboardRect.size.height, 0.0);

    myScrollView.contentInset= insets;
    myScrollView.scrollIndicatorInsets = insets;

    [myScrollView scrollRectToVisible:self.activeField.frame animated:YES];
}

-(void)keyboardWillHide:(NSNotification*)notification  {
    UIEdgeInsets insets = UIEdgeInsetsMake(0.0, 0.0, 0.0, 0.0);
    
    UIScrollView* myScrollView;
    for (UIView *i in self.view.subviews){
        if([i isKindOfClass:[UIScrollView class]]){
            myScrollView = (UIScrollView *)i;
        }
    }
    
    myScrollView.contentInset = insets;
    myScrollView.scrollIndicatorInsets = insets;
    [self.view endEditing:YES];
    
}


-(void)textFieldDidEndEditing:(UITextField *)sender
{
    self.activeField = nil;
}

-(BOOL)textFieldShouldReturn:(UITextField*)textField
{
    NSInteger nextTag = textField.tag + 1;
    // Try to find next responder
    UIResponder* nextResponder = [textField.superview viewWithTag:nextTag];
    if (nextResponder) {
        // Found next responder, so set it.
        [nextResponder becomeFirstResponder];
    } else {
        // Not found, so remove keyboard.
        [textField resignFirstResponder];
    }
    return NO; // We do not want UITextField to insert line-breaks.
}

//method to move the view up/down whenever the keyboard is shown/dismissed
-(void)setViewMovedUp:(BOOL)movedUp
{
    //    [UIView beginAnimations:nil context:NULL];
    //    [UIView setAnimationDuration:0.3]; // if you want to slide up the view
    //
    //    CGRect rect = self.view.frame;
    //    if (movedUp)
    //    {
    //        // 1. move the view's origin up so that the text field that will be hidden come above the keyboard
    //        // 2. increase the size of the view so that the area behind the keyboard is covered up.
    //        rect.origin.y -= kOFFSET_FOR_KEYBOARD;
    //        rect.size.height += kOFFSET_FOR_KEYBOARD;
    //    }
    //    else
    //    {
    //        // revert back to the normal state.
    //        rect.origin.y += kOFFSET_FOR_KEYBOARD;
    //        rect.size.height -= kOFFSET_FOR_KEYBOARD;
    //    }
    //    self.view.frame = rect;
    //
    //    [UIView commitAnimations];
}


- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    // register for keyboard notifications
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardWillShow:)
                                                 name:UIKeyboardWillShowNotification
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardWillHide:)
                                                 name:UIKeyboardWillHideNotification
                                               object:nil];
}

- (void)viewWillDisappear:(BOOL)animated
{
    // unregister for keyboard notifications while not visible.
    [[NSNotificationCenter defaultCenter] removeObserver:self
                                                    name:UIKeyboardWillShowNotification
                                                  object:nil];
    
    [[NSNotificationCenter defaultCenter] removeObserver:self
                                                    name:UIKeyboardWillHideNotification
                                                  object:nil];
    [super viewWillDisappear:animated];
}

@end
